@@ -15,19 +15,20 @@
 from __future__ import annotations
 
 import functools
-
 from typing import Any, Callable, Sequence
 
+import numpy as np
+
 from jax import tree_util
+from jax.interpreters import batching
+from jax.interpreters import mlir
+
 from jax._src import core
 from jax._src import dtypes
 from jax._src import util
 from jax._src import dispatch
+from jax._src.interpreters import ad
 from jax._src.lib import xla_client as xc
-from jax.interpreters import ad
-from jax.interpreters import batching
-from jax.interpreters import mlir
-import numpy as np
 
 # `pure_callback_p` is the main primitive for staging out Python pure callbacks.
 pure_callback_p = core.Primitive("pure_callback")
