@@ -17,9 +17,7 @@ import operator
 from typing import Callable, Optional
 
 import jax
-from jax._src import linear_util as lu
 from jax import tree_util
-from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters.batching import not_mapped
 from jax.interpreters import mlir
@@ -29,10 +27,12 @@ from jax.tree_util import (tree_flatten, tree_map, tree_structure,
                            tree_unflatten, treedef_tuple)
 from jax._src import core
 from jax._src import custom_api_util
+from jax._src import linear_util as lu
 from jax._src import source_info_util
 from jax._src import traceback_util
 from jax._src import util
 from jax._src.api_util import flatten_fun_nokwargs
+from jax._src.interpreters import ad
 
 
 source_info_util.register_exclusion(__file__)
