@@ -1,4 +1,4 @@
-# Copyright 2021 Google LLC
+# Copyright 2021 The JAX Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -37,9 +37,23 @@ FLAGS = flags.FLAGS
 
 
 flags.DEFINE_string(
-    'jax_iree_backend', os.getenv('JAX_IREE_BACKEND', 'dylib'),
+    'jax_iree_backend', os.getenv('JAX_IREE_BACKEND', 'cpu'),
     'IREE compiler backend to use.')
 
+iree_compiler_map = {
+  "cpu" : "llvm-cpu",
+  "cuda" : "cuda",
+  "vmvx" : "vmvx",
+  "vulkan" : "vulkan-spirv"
+}
+
+iree_runtime_map = {
+  "cpu" : "local-task",
+  "cuda" : "cuda",
+  "vmvx" : "local-task",
+  "vulkan" : "vulkan"
+}
+
 class IreeDevice:
 
   def __init__(self, client):
@@ -74,7 +88,7 @@ class IreeBuffer(xla_client.DeviceArrayBase):
   def copy_to_device(self, device):
     return self
 
-  def to_py(self) -> np.ndarray:
+  def __array__(self, dtype=None, context=None):
     return np.asarray(self._buffer)
 
   def to_iree(self):
@@ -90,8 +104,14 @@ class IreeBuffer(xla_client.DeviceArrayBase):
     return self  # no async
 
   # overrides repr on base class which expects _value and aval attributes
-  def __repr__(self):
-    return f'IreeBuffer({self.to_py()})'
+  def __repr__(self): return f'IreeBuffer({np.asarray(self)})'
+
+  @property
+  def _value(self):
+    return np.asarray(self)
+
+  def copy_to_host_async(self):
+    return self
 
 class IreeExecutable:
 
@@ -122,12 +142,14 @@ class IreeClient:
 
   def __init__(self,
                *,
-               runtime_driver: str = None):
+               iree_backend: Optional[str] = None):
     self.platform = "iree"
     self.platform_version = "0.0.1"
     self.runtime_type = "iree"
-    self.runtime_driver = (FLAGS.jax_iree_backend if runtime_driver is None
-                           else runtime_driver)
+    self.iree_backend = (FLAGS.jax_iree_backend if iree_backend is None
+                        else iree_backend)
+    self.compiler_driver = iree_compiler_map[self.iree_backend]
+    self.runtime_driver = iree_runtime_map[self.iree_backend]
     self.iree_config = iree.runtime.system_api.Config(self.runtime_driver)
     self._devices = [IreeDevice(self)]
 
@@ -162,12 +184,13 @@ class IreeClient:
     if platform.system() == "Darwin" and platform.machine() == "arm64":
       extra_args += ["--iree-llvm-target-triple=arm64-apple-darwin21.5.0"]
     iree_binary = iree.compiler.compile_str(
-        computation, target_backends=[self.runtime_driver], input_type="mhlo",
+        computation, target_backends=[self.compiler_driver], input_type="mhlo",
         # extended_diagnostics=True,
         extra_args=extra_args,
     )
     # Load it into the runtime.
-    vm_module = iree.runtime.VmModule.from_flatbuffer(iree_binary)
+    vm_module = iree.runtime.VmModule.from_flatbuffer(
+      self.iree_config.vm_instance, iree_binary)
     module_object = iree.runtime.load_vm_module(vm_module, self.iree_config)
     return IreeExecutable(self, self._devices, module_object, "main")
 
