@@ -18,13 +18,14 @@ from typing import Union, Sequence
 
 import numpy as np
 
+from jax import lax
+from jax.interpreters import batching
 from jax.interpreters import mlir
 from jax.interpreters import xla
-from jax import lax
-from jax.interpreters import ad
-from jax.interpreters import batching
+
 from jax._src.api import jit, linear_transpose, ShapeDtypeStruct
 from jax._src.core import Primitive, is_constant_shape
+from jax._src.interpreters import ad
 from jax._src.lib.mlir.dialects import hlo
 from jax._src.lib import xla_client
 from jax._src.lib import ducc_fft
@@ -113,12 +114,8 @@ def _fft_lowering_cpu(ctx, x, *, fft_type, fft_lengths):
   if any(not is_constant_shape(a.shape) for a in (ctx.avals_in + ctx.avals_out)):
     raise NotImplementedError("Shape polymorphism for custom call is not implemented (fft); b/261671778")
   x_aval, = ctx.avals_in
-  if xla_client.mlir_api_version < 41:
-    return [ducc_fft.ducc_fft_mhlo(x, x_aval.dtype, fft_type=fft_type,
-                                   fft_lengths=fft_lengths)]
-  else:
-    return [ducc_fft.ducc_fft_hlo(x, x_aval.dtype, fft_type=fft_type,
-                                  fft_lengths=fft_lengths)]
+  return [ducc_fft.ducc_fft_hlo(x, x_aval.dtype, fft_type=fft_type,
+                                fft_lengths=fft_lengths)]
 
 def _naive_rfft(x, fft_lengths):
   y = fft(x, xla_client.FftType.FFT, fft_lengths)
