@@ -18,18 +18,17 @@ import warnings
 
 import numpy as np
 
-from jax.interpreters import ad
+from jax import tree_util
 from jax.interpreters import batching
 from jax.interpreters import mlir
 from jax.interpreters import xla
 
-from jax import tree_util
-
 from jax._src import ad_util
 from jax._src import core
 from jax._src import dtypes
 from jax._src import util
 from jax._src.core import ShapedArray, ConcreteArray
+from jax._src.interpreters import ad
 from jax._src.lax import lax
 from jax._src.lax import convolution
 from jax._src.lax import slicing
