@@ -15,11 +15,10 @@
 import builtins
 from functools import partial
 import operator
-from typing import overload, Any, Callable, Optional, Sequence, Tuple, Union
+from typing import overload, Any, Callable, Literal, Optional, Sequence, Tuple, Union
 import warnings
 
 import numpy as np
-from typing_extensions import Literal
 
 from jax import core
 from jax import lax
@@ -612,7 +611,7 @@ def nanvar(a: ArrayLike, axis: Axis = None, dtype: DTypeLike = None, out: None =
   normalizer = sum(lax_internal.bitwise_not(lax_internal._isnan(a)),
                    axis=axis, keepdims=keepdims, where=where)
   normalizer = normalizer - ddof
-  normalizer_mask = lax.le(normalizer, 0)
+  normalizer_mask = lax.le(normalizer, lax_internal._zero(normalizer))
   result = sum(centered, axis, keepdims=keepdims, where=where)
   result = _where(normalizer_mask, np.nan, result)
   divisor = _where(normalizer_mask, 1, normalizer)
