@@ -82,7 +82,7 @@ def _scatter_impl(x, y, scatter_op, treedef, static_idx, dynamic_idx,
   dtype = lax.dtype(x)
   weak_type = dtypes.is_weakly_typed(x)
 
-  if dtype != dtypes.result_type(x, y):
+  if dtype != lax.dtype(y) and dtype != dtypes.result_type(x, y):
     # TODO(jakevdp): change this to an error after the deprecation period.
     warnings.warn("scatter inputs have incompatible types: cannot safely cast "
                   f"value from dtype={lax.dtype(y)} to dtype={lax.dtype(x)}. "
@@ -180,7 +180,7 @@ def _segment_update(name: str,
   out = jnp.full((num_buckets, num_segments) + data.shape[1:],
                  _get_identity(scatter_op, dtype), dtype=dtype)
   out = _scatter_update(
-    out, np.index_exp[lax.div(jnp.arange(segment_ids.shape[0]), bucket_size),
+    out, np.index_exp[jnp.arange(segment_ids.shape[0]) // bucket_size,
                       segment_ids[None, :]],
     data, scatter_op, indices_are_sorted,
     unique_indices, normalize_indices=False, mode=mode)
@@ -229,13 +229,13 @@ def segment_sum(data: Array,
     >>> data = jnp.arange(5)
     >>> segment_ids = jnp.array([0, 0, 1, 1, 2])
     >>> segment_sum(data, segment_ids)
-    DeviceArray([1, 5, 4], dtype=int32)
+    Array([1, 5, 4], dtype=int32)
 
     Using JIT requires static `num_segments`:
 
     >>> from jax import jit
     >>> jit(segment_sum, static_argnums=2)(data, segment_ids, 3)
-    DeviceArray([1, 5, 4], dtype=int32)
+    Array([1, 5, 4], dtype=int32)
   """
   return _segment_update(
       "segment_sum", data, segment_ids, lax.scatter_add, num_segments,
@@ -285,13 +285,13 @@ def segment_prod(data: Array,
     >>> data = jnp.arange(6)
     >>> segment_ids = jnp.array([0, 0, 1, 1, 2, 2])
     >>> segment_prod(data, segment_ids)
-    DeviceArray([ 0,  6, 20], dtype=int32)
+    Array([ 0,  6, 20], dtype=int32)
 
     Using JIT requires static `num_segments`:
 
     >>> from jax import jit
     >>> jit(segment_prod, static_argnums=2)(data, segment_ids, 3)
-    DeviceArray([ 0,  6, 20], dtype=int32)
+    Array([ 0,  6, 20], dtype=int32)
   """
   return _segment_update(
       "segment_prod", data, segment_ids, lax.scatter_mul, num_segments,
@@ -340,13 +340,13 @@ def segment_max(data: Array,
     >>> data = jnp.arange(6)
     >>> segment_ids = jnp.array([0, 0, 1, 1, 2, 2])
     >>> segment_max(data, segment_ids)
-    DeviceArray([1, 3, 5], dtype=int32)
+    Array([1, 3, 5], dtype=int32)
 
     Using JIT requires static `num_segments`:
 
     >>> from jax import jit
     >>> jit(segment_max, static_argnums=2)(data, segment_ids, 3)
-    DeviceArray([1, 3, 5], dtype=int32)
+    Array([1, 3, 5], dtype=int32)
   """
   return _segment_update(
       "segment_max", data, segment_ids, lax.scatter_max, num_segments,
@@ -395,13 +395,13 @@ def segment_min(data: Array,
     >>> data = jnp.arange(6)
     >>> segment_ids = jnp.array([0, 0, 1, 1, 2, 2])
     >>> segment_min(data, segment_ids)
-    DeviceArray([0, 2, 4], dtype=int32)
+    Array([0, 2, 4], dtype=int32)
 
     Using JIT requires static `num_segments`:
 
     >>> from jax import jit
     >>> jit(segment_min, static_argnums=2)(data, segment_ids, 3)
-    DeviceArray([0, 2, 4], dtype=int32)
+    Array([0, 2, 4], dtype=int32)
   """
   return _segment_update(
       "segment_min", data, segment_ids, lax.scatter_min, num_segments,
