@@ -24,18 +24,21 @@ import jax
 import jax.numpy as jnp
 from jax import lax
 from jax.config import config
-from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import mlir
 from jax.numpy.linalg import cholesky, svd, eigh
+
 from jax._src import core
 from jax._src import dtypes
 from jax._src import prng
 from jax._src.api import jit, vmap
 from jax._src.core import NamedShape
+from jax._src.interpreters import ad
 from jax._src.lax import lax as lax_internal
 from jax._src.lib import xla_bridge
-from jax._src.numpy.lax_numpy import _arraylike, _check_arraylike, _convert_and_clip_integer, _promote_dtypes_inexact
+from jax._src.numpy.lax_numpy import (
+    _arraylike, _check_arraylike, _convert_and_clip_integer,
+    _promote_dtypes_inexact)
 from jax._src.typing import Array, ArrayLike, DTypeLike
 from jax._src.util import prod, canonicalize_axis
 
