@@ -46,7 +46,7 @@ from jax._src import util
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import use_stablehlo
 from jax.interpreters import mlir
-from jax.interpreters import xla
+from jax._src.interpreters import xla
 
 
 source_info_util.register_exclusion(__file__)
@@ -175,6 +175,26 @@ class Lowering(Protocol):
     """
     raise NotImplementedError
 
+  def cost_analysis(self) -> Any:
+    """A summary of execution cost estimates.
+
+    Intended for visualization and debugging purposes. The object output by
+    this is some simple data structure that can easily be printed or serialized
+    (e.g. nested dicts, lists, and tuples with numeric leaves). However, its
+    structure can be arbitrary: it need not be consistent across versions of JAX
+    and jaxlib, or even across invocations. It is relayed directly to external
+    callers.
+
+    This function estimates execution cost in the absence of compiler
+    optimizations, which may drastically affect the cost. For execution cost
+    estimates after optimizations, compile this lowering and see
+    ``Compiled.cost_analysis``.
+
+    May raise ``NotImplementedError`` if unavailable, e.g. based on backend,
+    compiler, or runtime.
+    """
+    # TODO(frostig): improve annotation (arbitrary pytree)
+    raise NotImplementedError
 
 # -- Internal adapters from XLA-related objects to the above protocols
 
@@ -311,6 +331,8 @@ class XlaLowering(Lowering):
     else:
       raise ValueError(f"unknown dialect: {dialect}")
 
+  def cost_analysis(self) -> Dict[str, float]:
+    raise NotImplementedError("must override")
 
 
 # -- Public-facing API, plus helpers
@@ -539,7 +561,7 @@ class Lowered(Stage):
   def __init__(
       self,
       lowering: XlaLowering,
-      args_info,  # PyTreee of ArgInfo
+      args_info,  # PyTree of ArgInfo
       out_tree: tree_util.PyTreeDef,
       no_kwargs: bool = False):
     self._lowering = lowering
@@ -572,12 +594,15 @@ class Lowered(Stage):
 
   def compile(self) -> Compiled:
     """Compile, returning a corresponding ``Compiled`` instance."""
-    from jax.interpreters import pxla
+    from jax._src.interpreters import pxla
 
     if (jax.config.jax_array and
-        isinstance(self._lowering, pxla.MeshComputation) and
-        all(pxla._is_unspecified(o) for o in self._lowering.compile_args['out_shardings'])):
-      kw = dict(_allow_propagation_to_outputs=True)
+        isinstance(self._lowering, pxla.MeshComputation)):
+      kw = dict(
+          _allow_propagation_to_outputs=[
+              pxla._is_unspecified(o)
+              for o in self._lowering.compile_args["out_shardings"]]
+      )
     else:
       kw = {}
 
@@ -616,6 +641,24 @@ class Lowered(Stage):
     except NotImplementedError:
       return None
 
+  def cost_analysis(self) -> Optional[Any]:
+    """A summary of execution cost estimates.
+
+    Intended for visualization and debugging purposes. The object output by
+    this is some simple data structure that can easily be printed or serialized
+    (e.g. nested dicts, lists, and tuples with numeric leaves). However, its
+    structure can be arbitrary: it may be inconsistent across versions of JAX
+    and jaxlib, or even across invocations.
+
+    Returns ``None`` if unavailable, e.g. based on backend, compiler, or
+    runtime.
+    """
+    # TODO(frostig): improve annotation (basic pytree of arbitrary structure)
+    try:
+      return self._lowering.cost_analysis()
+    except NotImplementedError:
+      return None
+
 
 class Wrapped(Protocol):
   """A function ready to be specialized, lowered, and compiled.
