@@ -13,7 +13,8 @@
 # limitations under the License.
 """Module for state."""
 from jax._src.state.types import (ShapedArrayRef, ReadEffect, WriteEffect,
-                                  AccumEffect, StateEffect)
+                                  AccumEffect, StateEffect, RefEffect,
+                                  get_ref_state_effects)
 from jax._src.state.primitives import (ref_get, ref_set, ref_swap,
                                        ref_addupdate, get_p, swap_p,
                                        addupdate_p)
