@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from collections import namedtuple
 import functools
 from functools import partial, cached_property
 import itertools as it
@@ -260,8 +259,6 @@ def cache(max_size=4096):
 
 memoize = cache(max_size=None)
 
-CacheInfo = namedtuple("CacheInfo", ["hits", "misses", "maxsize", "currsize"])
-
 def weakref_lru_cache(call: Callable, maxsize=2048):
   """
   Least recently used cache decorator with weakref support.
@@ -406,20 +403,21 @@ def tuple_delete(t, idx):
 class HashableFunction:
   """Decouples function equality and hash from its identity.
 
-  Local lambdas and functiond defs are reallocated on each function call, making
+  Local lambdas and function defs are reallocated on each function call, making
   the functions created on different calls compare as unequal. This breaks our
   caching logic, which should really only care about comparing the semantics and
   not actual identity.
 
   This class makes it possible to compare different functions based on their
-  semantics. The parts that are taken into account are: the bytecode of
-  the wrapped function (which is cached by the CPython interpreter and is stable
-  across the invocations of the surrounding function), and `closure` which should
-  contain all values in scope that affect the function semantics. In particular
-  `closure` should contain all elements of the function closure, or it should be
-  possible to derive the relevant elements of the true function closure based
-  solely on the contents of the `closure` argument (e.g. in case some closed-over
-  values are not hashable, but are entirely determined by hashable locals).
+  semantics. The parts that are taken into account are: the bytecode of the
+  wrapped function (which is cached by the CPython interpreter and is stable
+  across the invocations of the surrounding function), and `closure` which
+  should contain all values in scope that affect the function semantics. In
+  particular `closure` should contain all elements of the function closure, or
+  it should be possible to derive the relevant elements of the true function
+  closure based solely on the contents of the `closure` argument (e.g. in case
+  some closed-over values are not hashable, but are entirely determined by
+  hashable locals).
   """
 
   def __init__(self, f, closure):
