@@ -91,13 +91,10 @@ from jax._src.core import (
   _check_jaxpr as _check_jaxpr,
   _check_map as _check_map,
   _compact_eqn_should_include as _compact_eqn_should_include,
-  _dim_handler_and_canonical as _dim_handler_and_canonical,
-  _dimension_handler_int as _dimension_handler_int,
   _dtype_object as _dtype_object,
   _effect_free_abstract_eval as _effect_free_abstract_eval,
   _encode_digits_alphabetic as _encode_digits_alphabetic,
   _forward_to_value as _forward_to_value,
-  _get_special_dim_handler as _get_special_dim_handler,
   _initialize_jax_jit_thread_local_state as _initialize_jax_jit_thread_local_state,
   _invalid_shape_error as _invalid_shape_error,
   _jaxpr_type_to_callable_annotation as _jaxpr_type_to_callable_annotation,
@@ -278,12 +275,3 @@ from jax._src.core import (
 )
 
 from typing import Any, Callable, Dict, Type
-
-# TODO(mattjj,frostig): remove these stubs (pytype workaround)
-extract_call_jaxpr: Callable
-eval_jaxpr_eqn: Callable
-initial_to_final_param_rules: Dict
-unit: Any
-abstract_unit: AbstractValue
-unitvar: Var
-UnitVar: Type
