@@ -27,13 +27,13 @@ from absl import flags
 
 import flax  # type: ignore[import]
 from flax import linen as nn
-import flax.optim
 
 import jax
 import jax.numpy as jnp
 
 from matplotlib import pyplot as plt  # type: ignore
 import numpy as np
+import optax
 import tensorflow as tf  # type: ignore
 import tensorflow_datasets as tfds  # type: ignore
 
@@ -232,10 +232,11 @@ class FlaxMNIST:
     return -jnp.mean(jnp.sum(predictions * labels, axis=1))
 
   @staticmethod
-  def update(optimizer, inputs, labels):
-    grad = jax.grad(FlaxMNIST.loss)(optimizer.target, inputs, labels)
-    optimizer = optimizer.apply_gradient(grad)
-    return optimizer
+  def update(tx, params, opt_state, inputs, labels):
+    grad = jax.grad(FlaxMNIST.loss)(params, inputs, labels)
+    updates, opt_state = tx.update(grad, opt_state)
+    params = optax.apply_updates(params, updates)
+    return params, opt_state
 
   @staticmethod
   def train(train_ds, test_ds, num_epochs, with_classifier=True):
@@ -253,20 +254,21 @@ class FlaxMNIST:
     momentum_mass = 0.9
 
     init_shape = jnp.ones((1,) + input_shape, jnp.float32)
-    initial_params = FlaxMNIST.model.init(rng, init_shape)["params"]
-    optimizer_def = flax.optim.Momentum(
-      learning_rate=step_size, beta=momentum_mass)
-    optimizer = optimizer_def.create(initial_params)
+    params = FlaxMNIST.model.init(rng, init_shape)["params"]
+    tx = optax.sgd(learning_rate=step_size, momentum=momentum_mass)
+    opt_state = tx.init(params)
 
     for epoch in range(num_epochs):
       start_time = time.time()
       for inputs, labels in tfds.as_numpy(train_ds):
-        optimizer = jax.jit(FlaxMNIST.update)(optimizer, inputs, labels)
+        params, opt_state = jax.jit(FlaxMNIST.update,
+                                    static_argnums=0)(tx, params, opt_state,
+                                                      inputs, labels)
       epoch_time = time.time() - start_time
       # Same accuracy function as for the pure JAX example
-      train_acc = PureJaxMNIST.accuracy(FlaxMNIST.predict, optimizer.target,
+      train_acc = PureJaxMNIST.accuracy(FlaxMNIST.predict, params,
                                         train_ds)
-      test_acc = PureJaxMNIST.accuracy(FlaxMNIST.predict, optimizer.target,
+      test_acc = PureJaxMNIST.accuracy(FlaxMNIST.predict, params,
                                        test_ds)
       logging.info("%s: Epoch %d in %0.2f sec", FlaxMNIST.name, epoch,
                    epoch_time)
@@ -278,7 +280,6 @@ class FlaxMNIST:
     # See discussion in README.md for packaging Flax models for conversion
     predict_fn = functools.partial(FlaxMNIST.predict,
                                    with_classifier=with_classifier)
-    params = optimizer.target
     return (predict_fn, params)
 
 
