@@ -20,14 +20,21 @@ file.
 Run this as follows:
 
 ```
-# Run all converters on all suites and regenerate the table.
+# Test all converters on all suites and regenerate the table.
 python3 models_test_main.py
 
-# Run only CNN and Resnet50 with the TFjs converter and don't write Markdown.
+# Test only CNN and Resnet50 with the TFjs converter and don't write Markdown.
 python3 models_test_main.py \
     --examples=flax/cnn,flax/resnet50 \
     --converters=jax2tf_noxla_tfjs \
     --write_markdown=False
+
+# Execute all CNN tests (including the tests for shape polymorphism):
+python3 models_test_main.py --example_prefix=flax/cnn
+
+# Execute only a specific CNN test for shape polymorphism (NOTE: quotes and
+# commas are stripped from the example names).
+python3 models_test_main.py --examples="flax/cnn_[(b ...)]"
 ```
 """
 from absl import app
