@@ -42,10 +42,11 @@ from jax.experimental.sparse.bcsr import BCSR
 from jax.experimental.sparse.coo import COO
 from jax.experimental.sparse.csr import CSR, CSC
 from jax.experimental.sparse.util import _coo_extract
-from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import mlir
+
 from jax._src import dtypes
+from jax._src.interpreters import ad
 from jax._src.typing import Array, DTypeLike, Shape
 
 
