@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 """
 Sparsify transform
@@ -52,6 +53,7 @@ from typing import (
 
 import numpy as np
 
+import jax
 from jax import core
 from jax import lax
 from jax._src import linear_util as lu
@@ -68,9 +70,10 @@ from jax._src.config import config
 from jax._src.lax.control_flow import _check_tree_and_avals
 from jax._src.numpy import lax_numpy
 from jax.experimental import sparse
-from jax.experimental.sparse import BCOO
+from jax.experimental.sparse import BCOO, BCSR
 
-sparse_rules : Dict[core.Primitive, Callable] = {}
+sparse_rules_bcoo : Dict[core.Primitive, Callable] = {}
+sparse_rules_bcsr : Dict[core.Primitive, Callable] = {}
 
 _zero_preserving_linear_unary_primitives = [
   lax.copy_p,
@@ -142,25 +145,31 @@ class SparsifyEnv:
     self._buffers.append(jnp.asarray(arr))  # type: ignore
     return len(self._buffers) - 1
 
-  def data(self, spvalue: 'SparsifyValue') -> Array:
+  def data(self, spvalue: SparsifyValue) -> Array:
     """Get the data buffer associated with a SparsifyValue."""
     if spvalue.data_ref is None:
       raise RuntimeError("Internal: requested data from spvalue with data_ref=None")
     return self._buffers[spvalue.data_ref]
 
-  def indices(self, spvalue: 'SparsifyValue') -> Array:
+  def indices(self, spvalue: SparsifyValue) -> Array:
     """Get the indices buffer associated with a SparsifyValue."""
     if spvalue.indices_ref is None:
       raise RuntimeError("Internal: requested indices from spvalue with indices_ref=None")
     return self._buffers[spvalue.indices_ref]
 
+  def indptr(self, spvalue: SparsifyValue) -> Array:
+    """Get the BCSR indptr buffer associated with a SparsifyValue."""
+    if spvalue.indptr_ref is None:
+      raise RuntimeError("Internal: requested indices from spvalue with indptr_ref=None")
+    return self._buffers[spvalue.indptr_ref]
+
   def dense(self, data):
     """Add a new dense array to the sparsify environment."""
-    return SparsifyValue(np.shape(data), self._push(data), None)
+    return SparsifyValue(np.shape(data), self._push(data))
 
-  def sparse(self, shape, data=None, indices=None,
-             *, data_ref=None, indices_ref=None, indices_sorted=False,
-             unique_indices=False):
+  def sparse(self, shape, data=None, indices=None, indptr=None,
+             *, data_ref=None, indices_ref=None, indptr_ref=None,
+             indices_sorted=False, unique_indices=False):
     """Add a new sparse array to the sparsify environment."""
     if data is not None:
       assert data_ref is None
@@ -174,14 +183,21 @@ class SparsifyEnv:
     else:
       assert indices_ref is not None and indices_ref < len(self._buffers)
 
-    return SparsifyValue(shape, data_ref, indices_ref, indices_sorted,
-                         unique_indices)
+    if indptr is not None:
+      assert indptr_ref is None
+      indptr_ref = self._push(indptr)
+    elif indptr_ref is not None:
+      assert indptr_ref < len(self._buffers)
+
+    return SparsifyValue(shape, data_ref, indices_ref, indptr_ref,
+                         indices_sorted=indices_sorted, unique_indices=unique_indices)
 
 
 class SparsifyValue(NamedTuple):
   shape: Tuple[int, ...]
   data_ref: Optional[int]
-  indices_ref: Optional[int]
+  indices_ref: Optional[int] = None
+  indptr_ref: Optional[int] = None
   indices_sorted: Optional[bool] = False
   unique_indices: Optional[bool] = False
 
@@ -195,8 +211,14 @@ class SparsifyValue(NamedTuple):
   def is_dense(self):
     return self.indices_ref is None
 
+  def is_bcoo(self):
+    return self.is_sparse() and self.indptr_ref is None
 
-_is_bcoo = lambda arg: isinstance(arg, BCOO)
+  def is_bcsr(self):
+    return self.is_sparse() and self.indptr_ref is not None
+
+
+_is_sparse_obj = lambda arg: isinstance(arg, (BCOO, BCSR))
 _is_spvalue = lambda arg: isinstance(arg, SparsifyValue)
 
 
@@ -210,9 +232,13 @@ def arrays_to_spvalues(
       return spenv.sparse(arg.shape, arg.data, arg.indices,
                           indices_sorted=arg.indices_sorted,
                           unique_indices=arg.unique_indices)
+    elif isinstance(arg, BCSR):
+      return spenv.sparse(arg.shape, arg.data, arg.indices, arg.indptr,
+                          indices_sorted=arg.indices_sorted,
+                          unique_indices=arg.unique_indices)
     else:
       return spenv.dense(arg)
-  return tree_map(array_to_spvalue, args, is_leaf=_is_bcoo)
+  return tree_map(array_to_spvalue, args, is_leaf=_is_sparse_obj)
 
 
 def spvalues_to_arrays(
@@ -221,11 +247,14 @@ def spvalues_to_arrays(
     ) -> Any:
   """Convert a pytree of spvalues to an equivalent pytree of (sparse) arrays."""
   def spvalue_to_array(spvalue):
-    if spvalue.is_sparse():
-      assert spvalue.indices_ref is not None
+    if spvalue.is_bcoo():
       return BCOO((spenv.data(spvalue), spenv.indices(spvalue)),
                   shape=spvalue.shape, indices_sorted=spvalue.indices_sorted,
                   unique_indices=spvalue.unique_indices)
+    elif spvalue.is_bcsr():
+      return BCSR((spenv.data(spvalue), spenv.indices(spvalue), spenv.indptr(spvalue)),
+                  shape=spvalue.shape, indices_sorted=spvalue.indices_sorted,
+                  unique_indices=spvalue.unique_indices)
     else:
       return spenv.data(spvalue)
   return tree_map(spvalue_to_array, spvalues, is_leaf=_is_spvalue)
@@ -293,9 +322,9 @@ class SparseTrace(core.Trace):
     spenv = popattr(self.main, 'spenv')
     spvalues = [t._spvalue for t in tracers]
     if any(spvalue.is_sparse() for spvalue in spvalues):
-      if primitive not in sparse_rules:
+      if primitive not in sparse_rules_bcoo:
         _raise_unimplemented_primitive(primitive)
-      out_spvalues = sparse_rules[primitive](spenv, *(t._spvalue for t in tracers), **params)
+      out_spvalues = sparse_rules_bcoo[primitive](spenv, *(t._spvalue for t in tracers), **params)
     else:
       out_bufs = primitive.bind(*(spenv.data(spvalue) for spvalue in spvalues), **params)
       out_spvalues = arrays_to_spvalues(spenv, out_bufs if primitive.multiple_results else [out_bufs])
@@ -340,7 +369,7 @@ def _sparsify_with_tracer(fun):
   """Implementation of sparsify() using tracers."""
   @functools.wraps(fun)
   def _wrapped(*args):
-    args_flat, in_tree = tree_flatten(args, is_leaf=_is_bcoo)
+    args_flat, in_tree = tree_flatten(args, is_leaf=_is_sparse_obj)
     wrapped_fun, out_tree = flatten_fun_nokwargs(lu.wrap_init(fun), in_tree)
     out = sparsify_fun(wrapped_fun, args_flat)
     return tree_unflatten(out_tree(), out)
@@ -382,11 +411,14 @@ def eval_sparse(
   for eqn in jaxpr.eqns:
     prim = eqn.primitive
     invals = safe_map(read, eqn.invars)
-
-    if any(val.is_sparse() for val in invals):
-      if prim not in sparse_rules:
+    if any(val.is_bcsr() for val in invals):
+      if prim not in sparse_rules_bcsr:
         _raise_unimplemented_primitive(prim)
-      out = sparse_rules[prim](spenv, *invals, **eqn.params)
+      out = sparse_rules_bcsr[prim](spenv, *invals, **eqn.params)
+    elif any(val.is_bcoo() for val in invals):
+      if prim not in sparse_rules_bcoo:
+        _raise_unimplemented_primitive(prim)
+      out = sparse_rules_bcoo[prim](spenv, *invals, **eqn.params)
     else:
       if prim is xla.xla_call_p:
         # TODO(vanderplas,frostig): workaround for binding call primitives
@@ -483,6 +515,7 @@ def _zero_preserving_unary_op(prim, linear):
     if spvalues[0].is_sparse():
       out_spvalue = spenv.sparse(spvalue.shape, buf_out,
                                  indices_ref=spvalue.indices_ref,
+                                 indptr_ref=spvalue.indptr_ref,
                                  indices_sorted=spvalue.indices_sorted,
                                  unique_indices=spvalue.unique_indices)
     else:
@@ -491,9 +524,11 @@ def _zero_preserving_unary_op(prim, linear):
   return func
 
 for _prim in _zero_preserving_unary_primitives:
-  sparse_rules[_prim] = _zero_preserving_unary_op(_prim, linear=False)
+  sparse_rules_bcoo[_prim] = _zero_preserving_unary_op(_prim, linear=False)
+  sparse_rules_bcsr[_prim] = _zero_preserving_unary_op(_prim, linear=False)
 for _prim in _zero_preserving_linear_unary_primitives:
-  sparse_rules[_prim] = _zero_preserving_unary_op(_prim, linear=True)
+  sparse_rules_bcoo[_prim] = _zero_preserving_unary_op(_prim, linear=True)
+  sparse_rules_bcsr[_prim] = _zero_preserving_unary_op(_prim, linear=True)
 
 def _standard_sparse_rule(prim, sparse_op):
   def _sparse_rule(spenv, *spvalues, **kwds):
@@ -504,15 +539,17 @@ def _standard_sparse_rule(prim, sparse_op):
 _BCOO_STANDARD_PRIMITIVES = {
   lax.broadcast_in_dim_p: sparse.bcoo_broadcast_in_dim,
   lax.concatenate_p: lambda *a, **k: sparse.bcoo_concatenate(a, **k),
+  lax.conv_general_dilated_p: sparse.bcoo_conv_general_dilated,
   lax.dot_general_p: sparse.bcoo_dot_general,
   lax.dynamic_slice_p: lambda *a, **k: sparse.bcoo_dynamic_slice(a[0], a[1:], **k),
   lax.reshape_p: sparse.bcoo_reshape,
+  lax.rev_p: sparse.bcoo_rev,
   lax.slice_p: sparse.bcoo_slice,
   lax.squeeze_p: sparse.bcoo_squeeze,
 }
 
 for prim, bcoo_impl in _BCOO_STANDARD_PRIMITIVES.items():
-  sparse_rules[prim] = _standard_sparse_rule(prim, bcoo_impl)
+  sparse_rules_bcoo[prim] = _standard_sparse_rule(prim, bcoo_impl)
 
 
 def _transpose_sparse(spenv, *spvalues, permutation):
@@ -546,7 +583,7 @@ def _transpose_sparse(spenv, *spvalues, permutation):
   spvalue = spenv.sparse(out_shape, **kwds)
   return (spvalue,)
 
-sparse_rules[lax.transpose_p] = _transpose_sparse
+sparse_rules_bcoo[lax.transpose_p] = _transpose_sparse
 
 def _add_sparse(spenv, *spvalues):
   X, Y = spvalues
@@ -572,16 +609,16 @@ def _add_sparse(spenv, *spvalues):
 
   return (out_spvalue,)
 
-sparse_rules[lax.add_p] = _add_sparse
+sparse_rules_bcoo[lax.add_p] = _add_sparse
 
 def _sub_sparse(spenv, *spvalues):
   X, Y = spvalues
   if X.is_sparse() and Y.is_sparse():
-    return _add_sparse(spenv, X, *sparse_rules[lax.neg_p](spenv, Y))
+    return _add_sparse(spenv, X, *sparse_rules_bcoo[lax.neg_p](spenv, Y))
   else:
     raise NotImplementedError("Subtraction between sparse and dense array.")
 
-sparse_rules[lax.sub_p] = _sub_sparse
+sparse_rules_bcoo[lax.sub_p] = _sub_sparse
 
 def _mul_sparse(spenv, *spvalues):
   X, Y = spvalues
@@ -609,7 +646,7 @@ def _mul_sparse(spenv, *spvalues):
 
   return (out_spvalue,)
 
-sparse_rules[lax.mul_p] = _mul_sparse
+sparse_rules_bcoo[lax.mul_p] = _mul_sparse
 
 def _reduce_sum_sparse(spenv, *spvalues, axes):
   X, = spvalues
@@ -622,7 +659,7 @@ def _reduce_sum_sparse(spenv, *spvalues, axes):
     out_spvalue = spenv.sparse(out_shape, mat.data, mat.indices)
   return (out_spvalue,)
 
-sparse_rules[lax.reduce_sum_p] = _reduce_sum_sparse
+sparse_rules_bcoo[lax.reduce_sum_p] = _reduce_sum_sparse
 
 
 
@@ -635,7 +672,7 @@ def _gather_sparse_rule(spenv, *args, dimension_numbers, slice_sizes, unique_ind
                               mode=mode, fill_value=fill_value)
   return arrays_to_spvalues(spenv, (result,))
 
-sparse_rules[lax.gather_p] = _gather_sparse_rule
+sparse_rules_bcoo[lax.gather_p] = _gather_sparse_rule
 
 def _sparsify_jaxpr(spenv, jaxpr, *spvalues):
   # TODO(jakevdp): currently this approach discards all information about
@@ -681,7 +718,7 @@ def _while_sparse(spenv, *spvalues, cond_jaxpr, cond_nconsts, body_jaxpr, body_n
                               body_nconsts=len(body_consts), body_jaxpr=body_sp_jaxpr)
   return arrays_to_spvalues(spenv, tree_unflatten(out_tree, out_flat))
 
-sparse_rules[lax.while_p] = _while_sparse
+sparse_rules_bcoo[lax.while_p] = _while_sparse
 
 def _xla_call_sparse(spenv, *spvalues, call_jaxpr, donated_invars, **params):
   if any(donated_invars):
@@ -693,7 +730,7 @@ def _xla_call_sparse(spenv, *spvalues, call_jaxpr, donated_invars, **params):
   out_flat = xla.xla_call_p.bind(fun, *args_flat, donated_invars=donated_invars, **params)
   return arrays_to_spvalues(spenv, tree_unflatten(out_tree, out_flat))
 
-sparse_rules[xla.xla_call_p] = _xla_call_sparse
+sparse_rules_bcoo[xla.xla_call_p] = _xla_call_sparse
 
 
 def _pjit_sparse(spenv, *spvalues, jaxpr, in_shardings, out_shardings,
@@ -733,7 +770,7 @@ def _pjit_sparse(spenv, *spvalues, jaxpr, in_shardings, out_shardings,
       inline=inline)
   return arrays_to_spvalues(spenv, tree_unflatten(out_tree, out_flat))
 
-sparse_rules[pjit.pjit_p] = _pjit_sparse
+sparse_rules_bcoo[pjit.pjit_p] = _pjit_sparse
 
 
 def _duplicate_for_sparse_spvalues(spvalues, params):
@@ -767,7 +804,7 @@ def _scan_sparse(spenv, *spvalues, jaxpr, num_consts, num_carry, **params):
   xs_out = tree_unflatten(xs_tree, out[len(carry):])
   return arrays_to_spvalues(spenv, carry_out + xs_out)
 
-sparse_rules[lax.scan_p] = _scan_sparse
+sparse_rules_bcoo[lax.scan_p] = _scan_sparse
 
 def _cond_sparse(spenv, pred, *operands, branches, linear, **params):
   sp_branches, treedefs = zip(*(_sparsify_jaxpr(spenv, jaxpr, *operands)
@@ -781,14 +818,14 @@ def _cond_sparse(spenv, pred, *operands, branches, linear, **params):
   out = tree_unflatten(treedefs[0], out_flat)
   return arrays_to_spvalues(spenv, out)
 
-sparse_rules[lax.cond_p] = _cond_sparse
+sparse_rules_bcoo[lax.cond_p] = _cond_sparse
 
 def _todense_sparse_rule(spenv, spvalue, *, tree):
   del tree  # TODO(jakvdp): we should assert that tree is PytreeDef(*)
   out = spvalues_to_arrays(spenv, spvalue).todense()
   return (spenv.dense(out),)
 
-sparse_rules[sparse.todense_p] = _todense_sparse_rule
+sparse_rules_bcoo[sparse.todense_p] = _todense_sparse_rule
 
 
 #------------------------------------------------------------------------------
