@@ -32,16 +32,15 @@ import numpy as np
 
 from jax._src import linear_util as lu
 from jax.config import config
-from jax.interpreters import ad
+from jax._src.interpreters import ad
 from jax.interpreters import partial_eval as pe
-from jax.interpreters import xla
+from jax._src.interpreters import xla
 from jax._src import ad_util
 from jax._src import core
 from jax._src import device_array
 from jax._src import dtypes
 from jax._src import source_info_util
 from jax._src import util
-from jax._src.lib import mlir_api_version
 from jax._src.lib import xla_bridge as xb
 from jax._src.lib import xla_client as xc
 from jax._src.lib.mlir import ir
@@ -124,11 +123,10 @@ _dtype_to_ir_type : Dict[np.dtype, Callable[[], ir.Type]] = {
   np.dtype(np.complex128): lambda: ir.ComplexType.get(ir.F64Type.get()),
 }
 
-if xc.mlir_api_version >= 43 and xc._version >= 113:
-  _dtype_to_ir_type.update({
-      np.dtype(dtypes.float8_e4m3fn): ir.Float8E4M3FNType.get,
-      np.dtype(dtypes.float8_e5m2): ir.Float8E5M2Type.get,
-  })
+_dtype_to_ir_type.update({
+    np.dtype(dtypes.float8_e4m3fn): ir.Float8E4M3FNType.get,
+    np.dtype(dtypes.float8_e5m2): ir.Float8E5M2Type.get,
+})
 
 def dtype_to_ir_type(dtype: Union[np.dtype, np.generic]) -> ir.Type:
   assert isinstance(dtype, (np.dtype, np.generic)), type(dtype)
@@ -143,7 +141,8 @@ def dtype_to_ir_type(dtype: Union[np.dtype, np.generic]) -> ir.Type:
 def _array_ir_types(aval: Union[core.ShapedArray, core.DShapedArray]
                     ) -> Sequence[ir.Type]:
   if core.is_opaque_dtype(aval.dtype):
-    return aval.dtype._rules.aval_to_ir_types(aval)
+    phys_avals = aval.dtype._rules.physical_avals(aval)
+    return tuple(itertools.chain(*map(_array_ir_types, phys_avals)))
   if not core.is_constant_shape(aval.shape):
     return _dynamic_array_ir_types(aval)  # type: ignore
   return (ir.RankedTensorType.get(aval.shape, dtype_to_ir_type(aval.dtype)),)
@@ -314,10 +313,7 @@ for t in device_array.device_array_types:
   register_constant_handler(t, _device_array_constant_handler)
 
 def _token_constant_handler(val, canonicalize_types):
-  if mlir_api_version < 40:
-    return [hlo.CreateTokenOp(hlo.TokenType.get()).result]
-  else:
-    return [hlo.CreateTokenOp().result]
+  return [hlo.CreateTokenOp().result]
 register_constant_handler(core.Token, _token_constant_handler)
 
 # Source locations
@@ -332,7 +328,7 @@ def _source_info_to_location(
   if frame is None:
     loc = ir.Location.unknown()
   else:
-    loc = ir.Location.file(xla._get_canonical_source_file(frame),
+    loc = ir.Location.file(xla.get_canonical_source_file(frame),
                            frame.start_line, frame.start_column)
   loc = ir.Location.name(eqn_str, childLoc=loc)
   # TODO(phawkins): also include primitive.name as the operator type.
@@ -541,6 +537,7 @@ def register_lowering(prim: core.Primitive, rule: LoweringRule,
     # this expansion.
     for p in xb.expand_platform_alias(platform):
       _platform_specific_lowerings[p][prim] = rule
+  return rule
 
 
 def _unwrap_singleton_ir_values(x): return x[0] if len(x) == 1 else x
@@ -582,28 +579,37 @@ def sharded_aval(aval: core.ShapedArray,
   return aval.update(tuple(sharded_shape))
 
 
-class DimPolyEvaluator:
+class DimExprEvaluator:
   # A wrapper for an ir.Value that overloads + and * to be used for evaluating
-  # dimension polynomials.
+  # symbolic dimensions.
+  __array_priority__ = 1000  # Same as tracer, for __radd__ and others on ndarray
   def __init__(self, value: ir.Value):
     self.value = value
 
-  def __add__(self, other: Union[np.int32, np.int64, DimPolyEvaluator]):
-    if not isinstance(other, DimPolyEvaluator):
-      other = DimPolyEvaluator(ir_constant(other))
-    return DimPolyEvaluator(hlo.AddOp(self.value, other.value).result)
+  def __add__(self, other: Union[np.int32, np.int64, DimExprEvaluator]):
+    if not isinstance(other, DimExprEvaluator):
+      other = DimExprEvaluator(ir_constant(other))
+    return DimExprEvaluator(hlo.AddOp(self.value, other.value).result)
 
   def __radd__(self, other: Union[np.int32, np.int64]):
-    return DimPolyEvaluator(hlo.AddOp(ir_constant(other), self.value).result)
+    return DimExprEvaluator(ir_constant(other)).__add__(self)
 
-  def __mul__(self, other: Union[np.int32, np.int64, DimPolyEvaluator]):
-    if not isinstance(other, DimPolyEvaluator):
-      other = DimPolyEvaluator(ir_constant(other))
-    return DimPolyEvaluator(hlo.MulOp(self.value, other.value).result)
+  def __mul__(self, other: Union[np.int32, np.int64, DimExprEvaluator]):
+    if not isinstance(other, DimExprEvaluator):
+      other = DimExprEvaluator(ir_constant(other))
+    return DimExprEvaluator(hlo.MulOp(self.value, other.value).result)
 
   def __rmul__(self, other: Union[np.int32, np.int64]):
-    return DimPolyEvaluator(hlo.MulOp(ir_constant(other), self.value).result)
+    return DimExprEvaluator(ir_constant(other)).__mul__(self)
 
+  def __divmod__(self, divisor: Union[np.int32, np.int64, DimExprEvaluator]):
+    if not isinstance(divisor, DimExprEvaluator):
+      divisor = DimExprEvaluator(ir_constant(divisor))
+    return (DimExprEvaluator(hlo.DivOp(self.value, divisor.value).result),
+            DimExprEvaluator(hlo.RemOp(self.value, divisor.value).result))
+
+  def __rdivmod__(self, dividend: Union[np.int32, np.int64]):
+    return DimExprEvaluator(ir_constant(dividend)).__divmod__(self)
 
 def eval_dynamic_shape(ctx: LoweringRuleContext,
                        shape: core.Shape) -> Tuple[Union[int, Value], ...]:
@@ -611,7 +617,7 @@ def eval_dynamic_shape(ctx: LoweringRuleContext,
   if config.jax_dynamic_shapes:
     return tuple(ctx.axis_size_env.get(d, d) for d in shape)  # type: ignore
   else:
-    dim_var_env = {dv_name : DimPolyEvaluator(dv_val[0])
+    dim_var_env = {dv_name : DimExprEvaluator(dv_val[0])
                    for dv_name, dv_val in zip(ctx.module_context.dim_vars, ctx.dim_var_values)}
     def eval_dim(d: core.DimSize) -> Union[int, ir.Value]:
       try:
@@ -668,6 +674,7 @@ def lower_jaxpr_to_module(
     for out_aval, out_sharding in zip(jaxpr.out_avals, result_shardings):
       if (out_aval is not core.abstract_token and
           core.is_opaque_dtype(out_aval.dtype)):
+        # TODO(frostig,mattjj,necula): asserts a single physical aval
         out_aval, = out_aval.dtype._rules.physical_avals(out_aval)
       out_avals.append(sharded_aval(out_aval, out_sharding))
 
@@ -770,11 +777,7 @@ def token_type() -> Sequence[ir.Type]:
   return [hlo.TokenType.get()]
 
 def create_token() -> Token:
-  if mlir_api_version < 40:
-    return wrap_singleton_ir_values(
-        hlo.CreateTokenOp(hlo.TokenType.get()).result)
-  else:
-    return wrap_singleton_ir_values(hlo.CreateTokenOp().result)
+  return wrap_singleton_ir_values(hlo.CreateTokenOp().result)
 
 class TokenSet:
   """An immutable container of tokens to be used to lower effectful jaxprs. When lowering
@@ -996,10 +999,7 @@ def lower_jaxpr_to_fun(
     args: List[List[ir.Value]] = []
     for aval, arg in zip(jaxpr.in_avals, unflattened_args):
       if replace_tokens_with_dummy and aval is core.abstract_token:
-        if mlir_api_version < 40:
-          args.append(hlo.CreateTokenOp(hlo.TokenType.get()).results)
-        else:
-          args.append(hlo.CreateTokenOp().results)
+        args.append(hlo.CreateTokenOp().results)
       else:
         args.append(arg)
     callee_name_stack = xla.extend_name_stack(ctx.name_stack,
@@ -1308,6 +1308,8 @@ def multi_broadcast_in_dim(ctx: LoweringRuleContext,
 
 def reshape(ctx: LoweringRuleContext, op, aval_out: core.AbstractValue) -> ir.Value:
   if core.is_opaque_dtype(aval_out.dtype):  # type: ignore
+    # TODO(frostig,mattjj,necula): asserts a single physical aval, and a
+    # particular reshape rule (reshape to the output physical aval's shape)
     aval_out, = aval_out.dtype._rules.physical_avals(aval_out)  # type: ignore
   if not core.is_constant_shape(aval_out.shape):  # type: ignore
     shape = eval_dynamic_shape(ctx, aval_out.shape)  # type: ignore
@@ -1365,11 +1367,7 @@ def dynamic_update_slice(ctx: LoweringRuleContext, aval_out, x, update, *,
         ctx, aval_out, x, update, *start_indices)
 
   # TODO(necula): handle dynamic shapes
-  if mlir_api_version < 40:
-    return hlo.DynamicUpdateSliceOp(
-        aval_to_ir_type(aval_out), x, update, start_indices).result
-  else:
-    return hlo.DynamicUpdateSliceOp(x, update, start_indices).result
+  return hlo.DynamicUpdateSliceOp(x, update, start_indices).result
 
 def pad(ctx: LoweringRuleContext, aval_out,
         x, padding_value,
@@ -1609,7 +1607,7 @@ def xla_fallback_lowering(prim: core.Primitive):
     return util.unflatten(flat_results, map(len, output_types))
   return fallback
 
-register_lowering(ad.custom_lin_p, ad._raise_custom_vjp_error_on_jvp)
+register_lowering(ad.custom_lin_p, ad.raise_custom_vjp_error_on_jvp)
 
 DEVICE_TO_DEVICE_TYPE = 1
 SEND_TO_HOST_TYPE = 2
@@ -1642,12 +1640,8 @@ def send_to_host(channel: int, token: hlo.TokenType, operand: Any,
                  aval: core.ShapedArray, name: str, *,
                  sharding: Optional[xc.OpSharding] = None) -> ir.Value:
   channel_handle = hlo.ChannelHandle.get(channel, SEND_TO_HOST_TYPE)
-  if mlir_api_version < 40:
-    send_op = hlo.SendOp(hlo.TokenType.get(), [operand], token, channel_handle,
-                         is_host_transfer=ir.BoolAttr.get(True))
-  else:
-    send_op = hlo.SendOp([operand], token, channel_handle,
-                         is_host_transfer=ir.BoolAttr.get(True))
+  send_op = hlo.SendOp([operand], token, channel_handle,
+                        is_host_transfer=ir.BoolAttr.get(True))
   dtype_str = _dtype_to_xla_type_string(aval.dtype)
   if dtype_str in {"f64", "s64", "u64", "c64", "c128"}:
     raise NotImplementedError("64-bit types not supported.")
@@ -1696,10 +1690,7 @@ def _emit_tpu_python_callback(
     *,
     sharding: Optional[xc.OpSharding] = None
 ) -> Tuple[List[ir.Value], Any, Any]:
-  if mlir_api_version < 40:
-    token = token or hlo.CreateTokenOp(hlo.TokenType.get()).result
-  else:
-    token = token or hlo.CreateTokenOp().result
+  token = token or hlo.CreateTokenOp().result
   _wrapped_callback = callback
 
   send_channels = []
