@@ -19,15 +19,21 @@ from jax._src.lax.linalg import (
   eig_p,
   eigh,
   eigh_p,
+  hessenberg,
+  hessenberg_p,
   lu,
   lu_p,
   lu_pivots_to_permutation,
+  householder_product,
+  householder_product_p,
   qr,
   qr_p,
   svd,
   svd_p,
   triangular_solve,
   triangular_solve_p,
+  tridiagonal,
+  tridiagonal_p,
   tridiagonal_solve,
   tridiagonal_solve_p,
   schur,
