@@ -37,5 +37,5 @@ from jax.scipy.stats import betabinom as betabinom
 from jax.scipy.stats import gennorm as gennorm
 from jax.scipy.stats import truncnorm as truncnorm
 from jax._src.scipy.stats.kde import gaussian_kde as gaussian_kde
-from jax._src.scipy.stats._core import mode as mode
+from jax._src.scipy.stats._core import mode as mode, rankdata as rankdata
 from jax.scipy.stats import vonmises as vonmises
